@@ -11,7 +11,7 @@ library(fbst)
 sleep
 
 ## -----------------------------------------------------------------------------
-library(BayesFactor)
+require(BayesFactor)
 grp1 = sleep[1:10,]$extra
 grp2 = sleep[11:20,]$extra
 ttestBF(x=grp1,y=grp2, rscale="medium")
@@ -27,7 +27,7 @@ result = fbet(posteriorDensityDraws = posteriorDraws, interval = c(-0.1,0.1), nu
 summary(result)
 
 ## ----fig.align='center', dpi=300, fig.width = 7, fig.height = 5, out.width = "500", out.height = "350"----
-plot(result)
+plot(result, type = "surprise")
 
 ## -----------------------------------------------------------------------------
 set.seed(42)
@@ -35,7 +35,7 @@ result2 = fbet(posteriorDensityDraws = posteriorDraws, interval = c(-0.2,0.2), n
 summary(result2)
 
 ## ----fig.align='center', dpi=300, fig.width = 7, fig.height = 5, out.width = "500", out.height = "350"----
-plot(result2)
+plot(result2, type = "surprise")
 
 ## -----------------------------------------------------------------------------
 set.seed(42)
@@ -44,13 +44,16 @@ result3 = fbet(posteriorDensityDraws = posteriorDraws, interval = c(-0.1,0.1),
 summary(result3)
 
 ## ----fig.align='center', dpi=300, fig.width = 7, fig.height = 5, out.width = "500", out.height = "350"----
-plot(result3)
+plot(result3, type="surprise")
 
 ## -----------------------------------------------------------------------------
 result4 = fbet(posteriorDensityDraws = posteriorDraws, interval = c(-0.2,0.2), 
                nu=1, FUN = dcauchy, par=list(location = 0, scale = sqrt(2)/2))
 summary(result4)
 
+## ----fig.align='center', dpi=300, fig.width = 7, fig.height = 5, out.width = "500", out.height = "350"----
+plot(result4, type="surprise")
+
 ## ----fig.align='center', dpi=300, fig.width = 7, fig.height = 5, out.width = "500", out.height = "350"----
 plot(result4)
 
